import { rescan } from "./modularThingClient.js";
import { global_state } from "./global_state.js";
import createSynchronizer from "../virtualThings/synchronizer";
import Terminal from "xterm";
import { render, html } from "lit-html";

import "./codemirror.js";

var xVal = 0;
var yVal = 0;
//can be manually adjusted
var xCenter = 50;
var yCenter = 40;


const view = (state) => html`
  <div class="menu">
    <div class="menu-item" @click=${runCode}>run (shift+enter)</div>
    <div class="menu-item" @click=${rescan}>scan</div>
    <div class="menu-item" @click=${() => global_state.viewWindow = !state.viewWindow}>view/code</div>
  </div>
  <div class="content">
    <div class="left-pane">
   
      <codemirror-editor></codemirror-editor>
      <div class="terminal">
        <div class="entry-line">\>\>\><input/></div>
      </div>
        
      <!-- <textarea spellcheck="false" class="code-editor"></textarea> -->
    </div>
    <div class="things">
      <div style="font-weight:700; font-size: 1.2em; padding-bottom: 10px;">List of Things</div>
      ${Object.entries(global_state.things).map(drawThing)}
    </div>

    <div class=${["view-window", state.viewWindow ? "" : "hide"].join(" ")}>Test</div>
  </div>
  ${state.renaming !== "" ? renameForm(state) : ""}
`

const drawThing = (thing) => html`
  <div class="thing">
    <div class="thing-top-line">
      <div class="thing-name">Name: ${thing[0]}</div>
      <button class="button" @click=${() => global_state.renaming = thing[0]}>rename</button>
    </div>
    <div>Type: ${thing[1].firmwareName}</div>
    <div class="thing-api">${drawApi(thing)}</div>
  </div>
  <hr/>
`

const drawApi = (thing) => {
  const [name, obj] = thing;
  const api = obj.vThing.api;



  return api.map(entry => html`
    <div class="apiEntry">
      <div>${entry.name}(${entry.args.map(x => x.split(":")[0]).join(", ")})</div>
      ${entry.args.map((x, i) => html`<div style="padding-left: 10px;">${x}</div>`)}
      ${entry.return
      ? html`<div class="apiEntry-return"><b>returns:</b> ${entry.return}</div>`
      : ""
    }
    </div>
  ` );
}

const getApi = (thing) => {
  const api = Object.keys(thing).map(x => [x, getParamNames(thing[x])]);
  // don't include "setup" or "setName" or "vt"
  return api
    .filter(x => !["setup", "setName", "vt", "firmwareName"].includes(x[0]))
    .map(apiEntry);
}

const apiEntry = ([name, params]) => html`
  <div class="apiEntry">${name}(${params.join(", ")})</div>
`

var STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
var ARGUMENT_NAMES = /([^\s,]+)/g;
function getParamNames(func) {
  var fnStr = func.toString().replace(STRIP_COMMENTS, '');
  var result = fnStr.slice(fnStr.indexOf('(') + 1, fnStr.indexOf(')')).match(ARGUMENT_NAMES);
  if (result === null)
    result = [];
  return result;
}

const renameForm = (state) => html`
  <div class="rename-form">
    <div class="rename-input">
      <div class="rename-label">Name:</div>
      <input .value=${state.renaming} class="rename-form-input"/>
    </div>
    <button class="button" @click=${() => {
    // console.log(state);
    const thing = state.things[state.renaming];
    const newName = document.querySelector(".rename-form-input").value;
    thing.vThing.setName(newName);
    delete state.things[state.renaming];
    state.things[newName] = thing;
    state.renaming = "";
  }}>rename</button>
    <button class="button" @click=${() => state.renaming = ""}>close</button>
  </div>
`

const r = () => {
  render(view(global_state), document.body);
  window.requestAnimationFrame(r);
};

window.state = () => console.log(global_state);

function init() {
  r();
  // "terminal is not a constructor" despite NPM reinstall 
  // var term = new Terminal();
  // term.open(document.querySelector('.terminal'));
  // term.write('Hello from \x1B[1;3;31mxterm.js\x1B[0m $ ');
  const cache = localStorage.getItem('cache');
  if (cache) {
    const cm = document.querySelector("codemirror-editor");
    cm.view.dispatch({
      changes: { from: 0, insert: cache ?? "" }
    });
  }


}

init();

function zhuoyueprint() {
  // console.log("hi this is zhuoyue!");
  var Module = {
    wasmBinaryFile: 'https://huningxin.github.io/opencv.js/build/wasm/opencv_js.wasm',
    preRun: [function () {
      Module.FS_createPreloadedFile('/', 'haarcascade_eye.xml', 'https://raw.githubusercontent.com/opencv/opencv/master/data/haarcascades/haarcascade_eye.xml', true, false);
      Module.FS_createPreloadedFile('/', 'haarcascade_frontalface_default.xml', 'https://raw.githubusercontent.com/opencv/opencv/master/data/haarcascades/haarcascade_frontalface_default.xml', true, false);
      Module.FS_createPreloadedFile('/', 'haarcascade_profileface.xml', 'https://raw.githubusercontent.com/opencv/opencv/master/data/haarcascades/haarcascade_profileface.xml', true, false);
    }],
    _main: function () { opencvIsReady(); }
  };
}

let videoWidth, videoHeight;

// whether streaming video from the camera.
let streaming = false;

let video = document.getElementById('video');
let canvasOutput = document.getElementById('canvasOutput');
let canvasOutputCtx = canvasOutput.getContext('2d');
let stream = null;

let detectFace = document.getElementById('face');
let detectEye = document.getElementById('eye');

function startCamera() {
  if (streaming) return;
  navigator.mediaDevices.getUserMedia({ video: true, audio: false })
    .then(function (s) {
      stream = s;
      video.srcObject = s;
      video.play();
    })
    .catch(function (err) {
      console.log("An error occured! " + err);
    });

  video.addEventListener("canplay", function (ev) {
    if (!streaming) {
      videoWidth = video.videoWidth;
      videoHeight = video.videoHeight;
      video.setAttribute("width", videoWidth);
      video.setAttribute("height", videoHeight);
      canvasOutput.width = videoWidth;
      canvasOutput.height = videoHeight;
      streaming = true;
    }
    startVideoProcessing();
  }, false);
}

let faceClassifier = null;
let eyeClassifier = null;

let src = null;
let dstC1 = null;
let dstC3 = null;
let dstC4 = null;

let canvasInput = null;
let canvasInputCtx = null;

let canvasBuffer = null;
let canvasBufferCtx = null;

function startVideoProcessing() {
  if (!streaming) { console.warn("Please startup your webcam"); return; }
  stopVideoProcessing();
  canvasInput = document.createElement('canvas');
  canvasInput.width = videoWidth;
  canvasInput.height = videoHeight;
  canvasInputCtx = canvasInput.getContext('2d');

  canvasBuffer = document.createElement('canvas');
  canvasBuffer.width = videoWidth;
  canvasBuffer.height = videoHeight;
  canvasBufferCtx = canvasBuffer.getContext('2d');

  srcMat = new cv.Mat(videoHeight, videoWidth, cv.CV_8UC4);
  grayMat = new cv.Mat(videoHeight, videoWidth, cv.CV_8UC1);

  faceClassifier = new cv.CascadeClassifier();
  faceClassifier.load('haarcascade_frontalface_default.xml');

  eyeClassifier = new cv.CascadeClassifier();
  eyeClassifier.load('haarcascade_eye.xml');

  requestAnimationFrame(processVideo);
}

function processVideo() {
  stats.begin();
  canvasInputCtx.drawImage(video, 0, 0, videoWidth, videoHeight);
  let imageData = canvasInputCtx.getImageData(0, 0, videoWidth, videoHeight);
  srcMat.data.set(imageData.data);
  cv.cvtColor(srcMat, grayMat, cv.COLOR_RGBA2GRAY);
  let faces = [];
  let eyes = [];
  let size;
  if (detectFace.checked) {
    let faceVect = new cv.RectVector();
    let faceMat = new cv.Mat();
    if (detectEye.checked) {
      cv.pyrDown(grayMat, faceMat);
      size = faceMat.size();
    } else {
      cv.pyrDown(grayMat, faceMat);
      cv.pyrDown(faceMat, faceMat);
      size = faceMat.size();
    }
    faceClassifier.detectMultiScale(faceMat, faceVect);
    for (let i = 0; i < faceVect.size(); i++) {
      let face = faceVect.get(i);
      faces.push(new cv.Rect(face.x, face.y, face.width, face.height));
      // console.log("Zhuoyue-X");
      // console.log(face.x);
      // console.log(face.y);
      if (detectEye.checked) {
        let eyeVect = new cv.RectVector();
        let eyeMat = faceMat.getRoiRect(face);
        eyeClassifier.detectMultiScale(eyeMat, eyeVect);
        for (let i = 0; i < eyeVect.size(); i++) {
          let eye = eyeVect.get(i);
          eyes.push(new cv.Rect(face.x + eye.x, face.y + eye.y, eye.width, eye.height));
        }
        eyeMat.delete();
        eyeVect.delete();
      }
    }
    faceMat.delete();
    faceVect.delete();
  } else {
    if (detectEye.checked) {
      let eyeVect = new cv.RectVector();
      let eyeMat = new cv.Mat();
      cv.pyrDown(grayMat, eyeMat);
      size = eyeMat.size();
      eyeClassifier.detectMultiScale(eyeMat, eyeVect);
      for (let i = 0; i < eyeVect.size(); i++) {
        let eye = eyeVect.get(i);
        eyes.push(new cv.Rect(eye.x, eye.y, eye.width, eye.height));
      }
      eyeMat.delete();
      eyeVect.delete();
    }
  }
  canvasOutputCtx.drawImage(canvasInput, 0, 0, videoWidth, videoHeight);
  drawResults(canvasOutputCtx, faces, 'red', size);
  drawResults(canvasOutputCtx, eyes, 'yellow', size);
  stats.end();
  requestAnimationFrame(processVideo);
}

function drawResults(ctx, results, color, size) {
  for (let i = 0; i < results.length; ++i) {
    let rect = results[i];
    let xRatio = videoWidth / size.width;
    let yRatio = videoHeight / size.height;
    ctx.lineWidth = 3;
    ctx.strokeStyle = color;
    ctx.strokeRect(rect.x * xRatio, rect.y * yRatio, rect.width * xRatio, rect.height * yRatio);
  }
}

function stopVideoProcessing() {
  if (src != null && !src.isDeleted()) src.delete();
  if (dstC1 != null && !dstC1.isDeleted()) dstC1.delete();
  if (dstC3 != null && !dstC3.isDeleted()) dstC3.delete();
  if (dstC4 != null && !dstC4.isDeleted()) dstC4.delete();
}

function stopCamera() {
  if (!streaming) return;
  stopVideoProcessing();
  document.getElementById("canvasOutput").getContext("2d").clearRect(0, 0, width, height);
  video.pause();
  video.srcObject = null;
  stream.getVideoTracks()[0].stop();
  streaming = false;
}

function initUI() {
  stats = new Stats();
  stats.showPanel(0);
  document.getElementById('container').appendChild(stats.dom);
}

function opencvIsReady() {
  console.log('OpenCV.js is ready');
  initUI();
  startCamera();
}

function getCode() {
  const cm = document.querySelector("codemirror-editor");
  const doc = cm.view.state.doc;
  const code = doc.toString();

  return code;
}

let intervals = [];
let timeouts = [];
let loops = [];



function loopFunction(delay, callback) {
  var loop = function () {
    callback();
    setTimeout(loop, delay);
  }; loop();
};


function runCode(e) {
  loopFunction(100, function () {
    var code = getCode();
    // console.log("check the code");
    zhuoyueprint();
    // it's string!!! we can simply provide code here I think
    // console.log(typeof code);
    // console.log(code);
    xVal = parseInt(document.getElementById("face-x-val-zy").innerHTML);
    yVal = parseInt(document.getElementById("face-y-val-zy").innerHTML);
    var actionlabelx = document.getElementById("face-actionx-val-zy");
    var actionlabely = document.getElementById("face-actiony-val-zy");
    // console.log('inside index.js');
    // console.log(xVal);
    // console.log(yVal);
    // console.log(typeof xVal);
    // console.log("check mediaDevices");
    // // console.log(navigator.mediaDevices.getUserMedia({ video: true, audio: false }));
    // console.log(navigator.mediaDevices);
    // console.log(navigator.mediaDevices.enumerateDevices());

    if (xVal - xCenter >= 0) {
      actionlabelx.innerHTML = "left";
      console.log("Left");
      // var testString = "aaa";
      code = code.replace("let direction = -1;", "let direction = 1;");
      // console.log(code);
    } else {
      actionlabelx.innerHTML = "right";
      console.log("Right");
      code= code.replace("let direction = 1;", "let direction = -1;");
      // console.log(code);
    }


    if (yVal - yCenter >= 0) {
      actionlabely.innerHTML = "up";
    } else {
      actionlabely.innerHTML = "down";
    }

    const AsyncFunction = (async function () { }).constructor;

    intervals.forEach(clearInterval);
    timeouts.forEach(clearTimeout);
    loops.forEach((x, i) => { loops[i] = false });


    const patchedInterval = (callback, time, ...args) => {
      const interval = setInterval(callback, time, ...args);
      intervals.push(interval);
      return interval;
    }

    const patchedTimeout = (callback, time, ...args) => {
      const timeout = setTimeout(callback, time, ...args);
      timeouts.push(timeout);
      return timeout;
    }

    const sleep = (ms) => new Promise(resolve => setTimeout(resolve, ms));

    const loop = async (fn, minterval = 0) => {
      let n = loops.length;
      loops.push(true);
      while (loops[n]) {
        const date = new Date();
        const start = date.getTime();
        await fn();
        const elapsed = (date.getTime()) - start;
        if (elapsed < minterval) await sleep(minterval - elapsed);
      }
    }

    const render = (node) => {
      const viewWindow = document.querySelector(".view-window");
      viewWindow.innerHTML = "";
      viewWindow.append(node);
    }

    const things = {};

    for (const key in global_state.things) {
      things[key] = global_state.things[key].vThing;
    }

    const args = {
      ...things,
      setInterval: patchedInterval,
      setTimeout: patchedTimeout,
      createSynchronizer,
      loop,
      render,
      // document: null,
      window: null,
      eval: null,
    }

    const names = Object.keys(args);
    const values = Object.values(args);

    const f = new AsyncFunction(
      ...names,
      code
    );

    f(
      ...values
    );




  });
}

window.addEventListener("keydown", (e) => {
  const code = getCode();

  window.localStorage.setItem("cache", code);

  if (e.keyCode === 13 && e.shiftKey) {
    runCode();
    e.preventDefault();
  }
})
