!(function () {
    // 老的浏览器可能根本没有实现 mediaDevices，所以我们可以先设置一个空的对象
    if (navigator.mediaDevices === undefined) {
        navigator.mediaDevices = {};
    }
    if (navigator.mediaDevices.getUserMedia === undefined) {
        navigator.mediaDevices.getUserMedia = function (constraints) {
            // 首先，如果有getUserMedia的话，就获得它
            var getUserMedia = navigator.webkitGetUserMedia || navigator.mozGetUserMedia || navigator.msGetUserMedia;

            // 一些浏览器根本没实现它 - 那么就返回一个error到promise的reject来保持一个统一的接口
            if (!getUserMedia) {
                return Promise.reject(new Error('getUserMedia is not implemented in this browser'));
            }

            // 否则，为老的navigator.getUserMedia方法包裹一个Promise
            return new Promise(function (resolve, reject) {
                getUserMedia.call(navigator, constraints, resolve, reject);
            });
        }
    }
    const constraints = {
        video: true,
        audio: false
    };
    let promise = navigator.mediaDevices.getUserMedia(constraints);
    promise.then(stream => {
        let v = document.getElementById('v');
        // 旧的浏览器可能没有srcObject
        if ("srcObject" in v) {
            v.srcObject = stream;
        } else {
            // 防止再新的浏览器里使用它，应为它已经不再支持了
            v.src = window.URL.createObjectURL(stream);
        }
        v.onloadedmetadata = function (e) {
            v.play();
        };
    }).catch(err => {
        console.error(err.name + ": " + err.message);
    })
})();

let src = new cv.Mat(v.height, v.width, cv.CV_8UC4);
let dst = new cv.Mat(v.height, v.width, cv.CV_8UC1);
let gray = new cv.Mat();
let cap = new cv.VideoCapture(v);
let faces = new cv.RectVector();
let classifier = new cv.CascadeClassifier();
let utils = new Utils('errorMessage');
let faceCascadeFile = 'haarcascade_frontalface_default.xml'; // path to xml
utils.createFileFromUrl(faceCascadeFile, faceCascadeFile, () => {
classifier.load(faceCascadeFile); // in the callback, load the cascade from file 
});
const FPS = 24;
function processVideo() {
    let begin = Date.now();
    cap.read(src);
    src.copyTo(dst);
    cv.cvtColor(dst, gray, cv.COLOR_RGBA2GRAY, 0);
    try{
        classifier.detectMultiScale(gray, faces, 1.1, 3, 0);
        console.log(faces.size());
    }catch(err){
        console.log(err);
    }
    for (let i = 0; i < faces.size(); ++i) {
        let face = faces.get(i);
        let point1 = new cv.Point(face.x, face.y);
        let point2 = new cv.Point(face.x + face.width, face.y + face.height);
        cv.rectangle(dst, point1, point2, [255, 0, 0, 255]);
    }
    cv.imshow("canvas_output", dst);
    // schedule next one.
    let delay = 1000/FPS - (Date.now() - begin);
    setTimeout(processVideo, delay);
}
// schedule first one.
setTimeout(processVideo, 0);
// let src = new cv.Mat(v.height, v.width, cv.CV_8UC4);
// let dst = new cv.Mat(v.height, v.width, cv.CV_8UC1);
// let gray = new cv.Mat();
// let cap = new cv.VideoCapture(v);
// let faces = new cv.RectVector();
// let classifier = new cv.CascadeClassifier();
// let utils = new Utils('errorMessage');
// let faceCascadeFile = 'haarcascade_frontalface_default.xml'; // path to xml
// utils.createFileFromUrl(faceCascadeFile, faceCascadeFile, () => {
// classifier.load(faceCascadeFile); // in the callback, load the cascade from file 
// });


// while cap.isOpened():
//     ret, frame = cap.read()
//     frame = cv.flip(frame, 1)  # mirror the image
//     gray = cv.cvtColor(frame, cv2.COLOR_BGR2GRAY)
//     faces = face_cascade.detectMultiScale(gray, 1.1, 6)  # detect the face
//     num_faces = len(faces)
//     string = '{0:d}'.format(num_faces)
//     print(string)
//     if num_faces != curr_num_faces:
//         # ArduinoSerial.write(string.encode('utf-8'))
//         curr_num_faces = num_faces
//     for x, y, w, h in faces:
//         cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 0, 255), 3)
//         string='X{0:d}Y{1:d}'.format((x+w//2),(y+h//2))
//         print(string)
//     cv2.imshow('img', frame)
//     # press q to Quit
//     if cv2.waitKey(10) & 0xFF == ord('q'):
//         break
// cap.release()
// cv2.destroyAllWindows()